@@ -20,7 +20,7 @@
  *  License along with this program; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Author: Darin Adler <darin@eazel.com>
+ *  Author: Darin Adler <darin@bentspoon.com>
  *
  */
 
@@ -30,6 +30,7 @@
 #include "nautilus-window.h"
 
 void                    nautilus_window_manage_views_destroy         (NautilusWindow         *window);
+void                    nautilus_window_manage_views_finalize        (NautilusWindow         *window);
 void                    nautilus_window_open_location                (NautilusWindow         *window,
                                                                       const char             *location);
 void                    nautilus_window_open_location_with_selection (NautilusWindow         *window,
